//    This file is part of Motion Watch.

//    Motion Watch is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.

//    Motion Watch is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.

#include "mo_detect.h"

MoDetect::MoDetect(shared_t *share, QObject *parent) : QVideoSink(parent)
{
    shared = share;

    connect(this, &MoDetect::videoFrameChanged, this, &MoDetect::newFrame);
}

void MoDetect::stop()
{
    baseImg = QImage(); gap = 0; shared->maxScore = 0;
}

void MoDetect::play(const QString &path)
{
    filePath = path;
}

void MoDetect::newFrame()
{
    if (!baseImg.isNull())
    {
        baseImg = videoFrame().toImage().convertToFormat(QImage::Format_Grayscale8);
    }
    else if (shared->frameGap > gap)
    {
        gap++;
    }
    else
    {
        auto nextImg     = videoFrame().toImage();
        auto nextImgGray = nextImg.convertToFormat(QImage::Format_Grayscale8);
        auto score   = 0;

        if (diff(baseImg, nextImgGray, &score))
        {
            shared->skipCmd = true;

            if (!shared->curEvent.srcPaths.contains(filePath))
            {
                shared->curEvent.srcPaths.append(filePath);

                if (shared->maxScore <= score)
                {
                    shared->maxScore = score;

                    shared->curEvent.thumbnail = nextImg;
                }
            }
        }
    }
}

bool MoDetect::diff(QImage &base, QImage &next, int *score)
{
    Mat baseMat = Mat(base.height(), base.width(), CV_8U, base.bits(), base.bytesPerLine());
    Mat nextMat = Mat(next.height(), next.width(), CV_8U, next.bits(), next.bytesPerLine());
    Mat diffMat;

    absdiff(baseMat, nextMat, diffMat);
    threshold(diffMat, diffMat, shared->pixThresh, 255, THRESH_BINARY);

    *score = countNonZero(diffMat);

    detLog("diff_score: " + QString::number(*score) + " thresh: " + QString::number(shared->imgThresh), shared);

    return *score >= shared->imgThresh;
}
